'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

var _formData = require('form-data');

var _formData2 = _interopRequireDefault(_formData);

var _urlSearchParams = require('url-search-params');

var _urlSearchParams2 = _interopRequireDefault(_urlSearchParams);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global FormData */

const MIME_TYPE_FORM = 'application/x-www-form-urlencoded';
const MIME_TYPE_JSON = 'application/json';
const MIME_TYPE_MULTIPART_FORM_DATA = 'multipart/form-data';

function createResponse(axiosResponse) {
  if (typeof axiosResponse !== 'object') {
    return axiosResponse;
  }

  return {
    status: axiosResponse.status,
    statusText: axiosResponse.statusText,
    headers: axiosResponse.headers,
    data: axiosResponse.data
  };
}

function createError(axiosError) {
  return {
    name: axiosError.name,
    code: axiosError.code,
    message: axiosError.message,
    response: createResponse(axiosError.response)
  };
}

_axios2.default.interceptors.response.use(response => createResponse(response), error => Promise.reject(createError(error)));

const methods = {
  create: 'POST',
  read: 'GET',
  update: 'PUT',
  delete: 'DELETE'
};

function toFormData(data) {
  const params = new _urlSearchParams2.default();

  if (data) {
    Object.keys(data).forEach(key => {
      const value = data[key];

      if (value !== null && typeof value !== 'undefined') {
        params.set(key, value);
      }
    });
  }

  return params.toString();
}

function toMultipartFormData(data) {
  const formData = typeof FormData !== 'undefined' ? new FormData() : new _formData2.default();

  if (data) {
    Object.keys(data).forEach(key => {
      const value = data[key];

      if (value !== null && typeof value !== 'undefined') {
        formData.append(key, value);
      }
    });
  }

  return formData;
}

function getConfig(requestConfig) {
  const url = requestConfig.resource;
  const method = methods[requestConfig.operation];
  const baseURL = requestConfig.url;
  const params = requestConfig.params;
  const responseType = requestConfig.responseType;

  let data = null;

  if (requestConfig.type === MIME_TYPE_FORM) {
    data = toFormData(requestConfig.data);
  } else if (requestConfig.type === MIME_TYPE_MULTIPART_FORM_DATA) {
    data = toMultipartFormData(requestConfig.data);
  } else {
    data = requestConfig.data;
  }

  const headers = {};

  let auth = null;

  if (_config2.default.hasOption(requestConfig, 'token')) {
    headers.Authorization = `Bearer ${requestConfig.token}`;
  } else if (requestConfig.username || requestConfig.password) {
    auth = {
      username: requestConfig.username,
      password: requestConfig.password
    };
  }

  if (_config2.default.hasOption(requestConfig, 'type')) {
    const type = requestConfig.type;

    if (type === MIME_TYPE_FORM || type === MIME_TYPE_JSON) {
      headers['Content-Type'] = `${type};charset=utf-8`;
    } else if (type === MIME_TYPE_MULTIPART_FORM_DATA) {
      if (data && data.getHeaders) {
        // If we're in node, using form-data, axios won't natively know what to do (it just sees a
        // stream). We need to manually set the content type header to have the correct mime type
        // and boundary, using the getHeaders method. If we're in a browser, using the built-in
        // FormData, the header will be set automatically, so we don't have to do anything.
        headers['Content-Type'] = data.getHeaders()['content-type'];
      }
    } else {
      headers['Content-Type'] = type;
    }
  }

  return {
    url,
    method,
    baseURL,
    headers,
    params,
    data,
    auth,
    responseType
  };
}

function sendRequest(requestConfig) {
  return (0, _axios2.default)(getConfig(requestConfig));
}

exports.default = {
  getConfig,
  sendRequest
};